@@ -21,7 +21,9 @@
 
 #include "galeon-embed-helper-list.h"
 #include "galeon-embed-persist.h"
+#ifdef ENABLE_NAUTILUS_VIEW
 #include "gul-bonobo-extensions.h"
+#endif
 #include "gul-string.h"
 #include "gul-general.h"
 
@@ -35,6 +37,12 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtkicontheme.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkiconfactory.h>
+#include <gtk/gtkmain.h>
 
 static void
 galeon_embed_helper_list_class_init (GaleonEmbedHelperListClass *klass);
@@ -59,6 +67,10 @@ enum
 	PROP_GALEON_EMBED
 };
 
+#define GALEON_EMBED_HELPER_LIST_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_EMBED_HELPER_LIST, GaleonEmbedHelperListPrivate))
+
+
 struct GaleonEmbedHelperListPrivate
 {
 	GaleonEmbed *embed;
@@ -120,12 +132,14 @@ galeon_embed_helper_list_class_init (GaleonEmbedHelperListClass *klass)
                                                               "Galeon Embed",
                                                               G_TYPE_OBJECT,
                                                               G_PARAM_READWRITE));
+
+	g_type_class_add_private (klass, sizeof (GaleonEmbedHelperListPrivate));
 }
 
 static void
 galeon_embed_helper_list_init (GaleonEmbedHelperList *dialog)
 {
-        dialog->priv = g_new0 (GaleonEmbedHelperListPrivate, 1);
+        dialog->priv = GALEON_EMBED_HELPER_LIST_GET_PRIVATE (dialog);
 
 	dialog->priv->embed = NULL;
 	dialog->priv->uri = NULL;
@@ -145,7 +159,7 @@ galeon_embed_helper_list_finalize (GObject *object)
         GaleonEmbedHelperList *dialog;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_GALEON_EMBED_HELPER_LIST (object));
+        g_return_if_fail (GALEON_IS_EMBED_HELPER_LIST (object));
 
         dialog = GALEON_EMBED_HELPER_LIST (object);
 
@@ -153,7 +167,6 @@ galeon_embed_helper_list_finalize (GObject *object)
 
 	g_free (dialog->priv->uri);
 	
-        g_free (dialog->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -193,7 +206,7 @@ galeon_embed_helper_list_get_property (GObject *object,
 GaleonEmbedHelperList *
 galeon_embed_helper_list_new (void)
 {
-	return GALEON_EMBED_HELPER_LIST (g_object_new (GALEON_EMBED_HELPER_LIST_TYPE, 
+	return GALEON_EMBED_HELPER_LIST (g_object_new (GALEON_TYPE_EMBED_HELPER_LIST, 
 						       NULL));
 }
 
@@ -229,6 +242,7 @@ galeon_embed_helper_list_set_uri (GaleonEmbedHelperList *hl,
 	}
 	else
 	{
+		g_free (mime);
 		return FALSE;
 	}
 
@@ -310,8 +324,8 @@ open_with_temp (GaleonEmbedHelperList *hl,
 	persist = galeon_embed_persist_new (hl->priv->embed);
 	galeon_embed_persist_set_source (persist, location);
 	galeon_embed_persist_set_dest (persist, final_filename);
-	galeon_embed_persist_set_handler (persist, app->command, 
-					  app->requires_terminal);
+	galeon_embed_persist_set_handler (persist, app);
+	galeon_embed_persist_set_user_time (persist, gtk_get_current_event_time());
 	galeon_embed_persist_save (persist);
 	g_object_unref (G_OBJECT (persist));
 
@@ -322,14 +336,26 @@ static gboolean
 application_supports_uri_scheme (GnomeVFSMimeApplication *application,
                                  const char *uri_scheme)
 {
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+
+	/* There is no way to get the supported schemes in the new API */
+	return gnome_vfs_mime_application_supports_uris (application);
+#else
+	if (application->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
+	{
+		return FALSE;
+	}
+
         /* The default supported uri scheme is "file" */
         if (application->supported_uri_schemes == NULL
             && g_ascii_strcasecmp (uri_scheme, "file") == 0) {
                 return TRUE;
         }
+
         return g_list_find_custom (application->supported_uri_schemes,
                                    uri_scheme,
                                    gul_strcasecmp_compare_func) != NULL;
+#endif
 }
 
 static gboolean
@@ -355,9 +381,9 @@ open_directly (const char *location,
 	
 	if (scheme && application_supports_uri_scheme (application, scheme))
 	{
-		gul_general_launch_application (application->command,
-						final_location,
-						application->requires_terminal);
+		/* FIXME: check return value */
+		gul_general_launch_application (application, final_location,
+						gtk_get_current_event_time());
 		result = TRUE;
 	}
 
@@ -367,6 +393,7 @@ open_directly (const char *location,
 	return result;
 }
 
+#ifdef ENABLE_NAUTILUS_VIEW
 static void
 bonobo_launch_application_callback (BonoboUIComponent *component, gpointer callback_data, const char *path)
 {
@@ -447,6 +474,7 @@ add_application_to_bonobo_menu (GaleonEmbedHelperList *hl,
                                 (GDestroyNotify) application_launch_parameters_free);
         g_free (tip);
 }
+#endif
 
 static void
 activate_cb (GtkMenuItem *mi, ApplicationLaunchParameters *launch_parameters)
@@ -460,53 +488,218 @@ activate_cb (GtkMenuItem *mi, ApplicationLaunchParameters *launch_parameters)
 	}
 }       
 
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+static GtkWidget*
+get_icon_from_application (GnomeVFSMimeApplication *application)
+{
+	GtkWidget *image = NULL;
+	const char *icon;
+	GtkIconTheme *theme;
+	char *name = NULL;
+
+	icon = gnome_vfs_mime_application_get_icon (application);
+	if (!icon)
+	{
+		return NULL;
+	}
+
+	if (g_path_is_absolute (icon))
+	{
+		int width, height;
+		GdkPixbuf *pixbuf;
+
+		if (!gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height))
+		{
+			width = height = 16;
+		}
+
+		pixbuf = gdk_pixbuf_new_from_file_at_size (icon, width, height, NULL);
+		if (pixbuf)
+		{
+			image = gtk_image_new_from_pixbuf (pixbuf);
+			g_object_unref (pixbuf);
+		}
+		
+		return image;
+	}
+
+	theme = gtk_icon_theme_get_default ();
+
+	if (gtk_icon_theme_has_icon (theme, icon))
+	{
+		name = g_strdup (icon);
+	}
+	else if (strrchr (icon, '.') != NULL)
+	{
+		/* If the image is foo.png then search for 'foo' in the icon theme
+		 * to allow themes to change the icon */
+		name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+		if (!gtk_icon_theme_has_icon (theme, name))
+		{
+			g_free (name);
+			return NULL;
+		}
+
+	}
+
+	if (name)
+	{
+		image = gtk_image_new_from_icon_name (name, GTK_ICON_SIZE_MENU);
+		g_free (name);
+	}
+
+	return image;
+}
+#endif
+
 static void
 add_application_to_gtk_menu (GaleonEmbedHelperList *hl,
 			     GtkMenuShell *ms,
 			     GnomeVFSMimeApplication *application, 
 			     const char *url)
 {
-        ApplicationLaunchParameters *launch_parameters;
 	GtkWidget *mi;
 
-        launch_parameters = application_launch_parameters_new (application, url, hl);
+	if (application != NULL)
+	{
+		ApplicationLaunchParameters *launch_parameters;
+
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+		const char *name;
+		GtkWidget *image;
+
+		name = gnome_vfs_mime_application_get_name (application);
+
+		mi = gtk_image_menu_item_new_with_label (name);
+
+		image = get_icon_from_application (application);
+		if (image)
+		{
+			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+		}
+#else
+		mi = gtk_menu_item_new_with_label (application->name);
+#endif
+
+		launch_parameters = application_launch_parameters_new (application, url, hl);
+		g_signal_connect_data (mi, "activate", 
+				       G_CALLBACK (activate_cb), 
+				       launch_parameters, (GClosureNotify) application_launch_parameters_free,
+				       (GConnectFlags) 0);
+	}
+	else
+	{
+		mi = gtk_separator_menu_item_new ();
+	}
 
-	mi = gtk_menu_item_new_with_label (application->name);
 	gtk_widget_show (mi);
 	gtk_menu_shell_append (ms, mi);
-	g_signal_connect_data (mi, "activate", 
-			       G_CALLBACK (activate_cb), 
-			       launch_parameters, (GClosureNotify) application_launch_parameters_free,
-			       (GConnectFlags) 0);
 }
 
+static int
+application_namecasecmp (gconstpointer ap, gconstpointer bp)
+{
+	const GnomeVFSMimeApplication *a = ap;
+	const GnomeVFSMimeApplication *b = bp;
+	char *a_name;
+	char *b_name;
+	int   ret;
+
+	a_name = g_utf8_casefold (a->name, -1);
+	b_name = g_utf8_casefold (b->name, -1);
+
+	ret = g_utf8_collate (a_name, b_name);
+
+	g_free (a_name);
+	g_free (b_name);
+
+	return ret;
+}
+
+static gboolean
+app_in_application_list (GnomeVFSMimeApplication *app, GList *all_apps)
+{
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+	GList *l;
+
+	for (l = all_apps ; l ; l = l->next)
+	{
+		if (gnome_vfs_mime_application_equal (app, l->data))
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+
+#else
+
+	return gnome_vfs_mime_id_in_application_list (app->id, all_apps);
+#endif
+}
+
+static GList *
+get_all_applications (GList *mime_types)
+{
+	GnomeVFSMimeApplication *default_app;
+	GList *all_apps = NULL;
+	GList *lm;
+
+	default_app = gnome_vfs_mime_get_default_application (mime_types->data);
+
+	for (lm = mime_types; lm != NULL; lm = lm->next)
+	{
+		const char *mime_type = lm->data;
+		GList      *apps;
+		GList      *la;
+
+		apps = gnome_vfs_mime_get_all_applications (mime_type);
+		for (la = apps; la != NULL; la = la->next)
+		{
+			GnomeVFSMimeApplication *app = la->data;
+			if (app_in_application_list (app, all_apps))
+			{
+				gnome_vfs_mime_application_free (app);
+			}
+			else if (default_app && strcmp (default_app->id, app->id))
+			{
+				all_apps = g_list_prepend (all_apps, app);
+			}
+		}
+		g_list_free (apps);
+	}
+
+	all_apps = g_list_sort (all_apps, application_namecasecmp);
+	if (default_app != NULL)
+	{
+		all_apps = g_list_prepend (all_apps, NULL);
+		all_apps = g_list_prepend (all_apps, default_app);
+	}
+	return all_apps;
+}
+
+#ifdef ENABLE_NAUTILUS_VIEW
 static void
 setup_openwith_list_bonoboui (GaleonEmbedHelperList *hl,
 			      BonoboUIComponent *ui_component,
 			      const char *path)
 {
-	int index = 0;
+	int    index = 0;
 	GList *apps;
-	GList *tl;
+	GList *l;
 	
-	for (tl = hl->priv->mime_types; tl != NULL; tl = tl->next)
+	apps = get_all_applications (hl->priv->mime_types);
+
+	for (l = apps; l != NULL; l = l->next)
 	{
-		GList *l;
-		
-		apps = gnome_vfs_mime_get_short_list_applications 
-			((char *)tl->data);
-		
-		for (l = apps; l != NULL; l = l->next)
-		{
-			add_application_to_bonobo_menu 
-				(hl, ui_component, 
-				 (GnomeVFSMimeApplication *)l->data,
-				 path, hl->priv->uri, index);
-			index ++;
-		}
-		
-		gnome_vfs_mime_application_list_free (apps);
+		add_application_to_bonobo_menu 
+			(hl, ui_component, 
+			 (GnomeVFSMimeApplication *)l->data,
+			 path, hl->priv->uri, index);
+		index ++;
 	}
+		
+	gnome_vfs_mime_application_list_free (apps);
 }
 	
 void
@@ -516,24 +709,23 @@ galeon_embed_helper_list_add_to_bonoboui_menu (GaleonEmbedHelperList *hl,
 {
 	setup_openwith_list_bonoboui (hl, ui_component, path);
 }
+#endif
+
 
 void
 galeon_embed_helper_list_add_to_gtk_menu (GaleonEmbedHelperList *hl,
 					  GtkMenuShell *ms)
 {
-	GList *tl;
-	
-	for (tl = hl->priv->mime_types; tl != NULL; tl = tl->next)
+	GList *apps;
+	GList *l;
+
+	apps = get_all_applications (hl->priv->mime_types);
+
+	for (l = apps; l != NULL; l = l->next)
 	{
-		GList *apps = gnome_vfs_mime_get_short_list_applications (tl->data);
-		GList *l;
-		
-		for (l = apps; l != NULL; l = l->next)
-		{
-			add_application_to_gtk_menu (hl, ms, l->data, hl->priv->uri);
-		}
-		
-		gnome_vfs_mime_application_list_free (apps);
+		add_application_to_gtk_menu (hl, ms, l->data, hl->priv->uri);
 	}
+
+	gnome_vfs_mime_application_list_free (apps);
 }
 
