@@ -20,6 +20,11 @@
 #include "config.h"
 #endif
 
+#define MOZILLA_INTERNAL_API
+#include <nscore.h>
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+
 #include "galeon-embed.h"
 #include "galeon-debug.h"
 #include "gul-string.h"
@@ -62,9 +67,6 @@
 #include <nsIDOMNSEvent.h>
 #include <nsIDOM3Document.h>
 #include <nsIScriptSecurityManager.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
 #include <nsIContentPolicy.h>
 #include <nsIDOMDocumentView.h>
 #include <nsIDOMAbstractView.h>
@@ -553,48 +555,6 @@ nsresult GaleonWrapper::Find (PRBool backwards,
 }
 
 
-#ifdef HAVE_GTKMOZEMBED_BROKEN_RELOAD
-/* Workaround for broken reload with frames, see mozilla bug
- * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
- */
-nsresult GaleonWrapper::ReloadPage (GtkMozEmbedReloadFlags aReloadFlags)
-{
-	nsresult rv;
-
-	nsCOMPtr<nsISHistory> sessionHistory;
-	rv = GetSHistory (getter_AddRefs (sessionHistory));
-	NS_ENSURE_SUCCESS (rv, rv);
-
-	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface (sessionHistory);
-
-	if (!wn)
-	{
-		wn = do_QueryInterface (mWebBrowser);
-	}
-
-	if (!wn) return NS_ERROR_FAILURE;
-	
-	PRUint32 reloadFlags = 0;
-	switch (aReloadFlags)
-	{
-	case GTK_MOZ_EMBED_FLAG_RELOADNORMAL: break;
-	case GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE:
-		reloadFlags = nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
-			nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY;
-		break;
-	case GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE:
-		reloadFlags = nsIWebNavigation::LOAD_FLAGS_CHARSET_CHANGE;
-		break;
-	case GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE:
-	case GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXY:
-		g_return_val_if_reached (NS_ERROR_FAILURE);
-		break;
-	}
-
-	return wn->Reload (reloadFlags);
-}
-#endif
-
 /* Don't use the Session History to reload a view source page,
  * it doesn't work:
  *
@@ -757,7 +717,7 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest,
 	if (count)
 	{
 		nsCOMPtr<nsIHistoryEntry> he;
-		nsCOMPtr<nsISHEntry> she;
+		nsCOMPtr<nsISHEntry> she, dhe;
 
 		for (PRInt32 i = (back_history ? 0 : index + 1); 
 		     i < (forward_history ? count : index + 1);
@@ -769,8 +729,11 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest,
 
 			she = do_QueryInterface (he);
 			NS_ENSURE_TRUE (she, NS_ERROR_FAILURE);
-			
-			rv = hi_dest->AddEntry (she, PR_TRUE);
+
+			rv = she->Clone (getter_AddRefs(dhe));
+			NS_ENSURE_SUCCESS (rv, rv);
+
+			rv = hi_dest->AddEntry (dhe, PR_TRUE);
 			NS_ENSURE_SUCCESS (rv, rv);
 		}
 		
@@ -1083,6 +1046,24 @@ nsresult GaleonWrapper::ScrollRight (void)
 	return DOMWindow->ScrollBy(16, 0);
 }
 
+nsresult GaleonWrapper::ScrollPageUp (void)
+{
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	NS_ENSURE_TRUE (DOMWindow, NS_ERROR_FAILURE);
+
+	return DOMWindow->ScrollByPages(-1);
+}
+
+nsresult GaleonWrapper::ScrollPageDown (void)
+{
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	NS_ENSURE_TRUE (DOMWindow, NS_ERROR_FAILURE);
+
+	return DOMWindow->ScrollByPages(1);
+}
+
 nsresult GaleonWrapper::FineScroll (int horiz, int vert)
 {
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
@@ -1280,7 +1261,10 @@ nsresult GaleonWrapper::GetPageProperties (EmbedPageProperties *props)
 	/* This might not work with XUL pages (e.g. about:config) */
 	// Until https://bugzilla.mozilla.org/show_bug.cgi?id=154359 is fixed.
 	rv = MozillaPrivate::GetCompatibilityMode(doc, &props->rendering_mode);
-	NS_ENSURE_SUCCESS(rv, rv);
+	if (NS_FAILED (rv))
+	{
+		props->rendering_mode = EMBED_RENDER_FULL_STANDARDS;
+	}
 
 	/* Get the URL so we can look in the cache for the page */
 	nsCOMPtr<nsIDOMLocation> DOMLocation;
