@@ -77,6 +77,7 @@
 
 #include <glib/gi18n.h>
 
+#include "AutoJSContextStack.h"
 #include "GtkNSSDialogs.h"
 #include "gul-glade.h"
 
@@ -209,6 +210,11 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label, *checkbox, *vbox;
 	int res;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
@@ -541,6 +547,11 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	GtkWidget *check_ssl, *check_software;
 	char *ttCommonName, *msg, *tertiary;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
@@ -622,7 +633,7 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check_software)))
 		{
-			*_trust |= nsIX509CertDB::TRUSTED_SSL;
+			*_trust |= nsIX509CertDB::TRUSTED_OBJSIGN;
 		}
 		*_retval = PR_TRUE;
 	}
@@ -758,6 +769,11 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *progress;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
@@ -869,6 +885,11 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *hbox, *label, *entry, *vbox;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
@@ -1293,6 +1314,10 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	PRUnichar ** usage;
 	GtkSizeGroup * sizegroup;
 
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	gxml = gul_glade_widget_new ("certificate-dialogs.glade", 
 				     "viewcert_dialog",
 				     &dialog, NULL);
