@@ -22,9 +22,12 @@
 #include <config.h>
 #endif
 
+#define MOZILLA_INTERNAL_API
+#include <nscore.h>
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsCOMPtr.h>
 #include <nsIIOService.h>
-#include <nsIServiceManager.h>
 #include <nsIURI.h>
 #include <nsIChannel.h>
 #include <nsIOutputStream.h>
@@ -444,7 +447,7 @@ GMyportalProtocolHandler::CreateMyportalPage (const char *path,
 	Write ("</head>\n");
 	Write ("<body>\n");
 	Write ("<div class=\"top_bar\">\n");
-	Write ("<img src=\"" FILE_PREFIX "file://" SHARE_DIR "/galeon.png\" />myportal:\n");
+	Write ("<img src=\"" FILE_PREFIX "file://" SHARE_DIR "/galeon.png\" alt=\"\" />myportal:\n");
 	Write ("</div>\n");
 	Write ("<div class=\"top2\">\n");
 	Write ("<a href=\"http://galeon.sourceforge.net\">");
@@ -454,7 +457,7 @@ GMyportalProtocolHandler::CreateMyportalPage (const char *path,
 	RenderFromPath (path);
 	Write ("\n\n<!-- END -->");
 	Write ("<div class=\"bottom_bar\">\n");
-	Write ("<center><b>Galeon</b> : <i>the web, only the web</i></center>\n");
+	Write ("<b>Galeon</b> : <i>the web, only the web</i>\n");
 	Write ("</div>");
 	Write ("</body></html>\n");
  
@@ -465,12 +468,16 @@ GMyportalProtocolHandler::CreateMyportalPage (const char *path,
 	rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
 	NS_ENSURE_SUCCESS (rv, rv);
 
-	nsCOMPtr<nsIInputStreamChannel> channel =
+	nsCOMPtr<nsIInputStreamChannel> isc =
 		do_CreateInstance(kInputStreamChannelCID, &rv);
 	NS_ENSURE_SUCCESS (rv, rv);
 
-        rv |= channel->SetURI (aURI);
-	rv |= channel->SetContentStream (iStream);
+        rv |= isc->SetURI (aURI);
+	rv |= isc->SetContentStream (iStream);
+
+	nsCOMPtr<nsIChannel> channel = do_QueryInterface (isc, &rv);
+	NS_ENSURE_SUCCESS (rv, rv);
+
         rv |= channel->SetContentType (NS_LITERAL_CSTRING ("application/xhtml+xml"));
         rv |= channel->SetContentCharset (NS_LITERAL_CSTRING ("utf-8"));
 	NS_ENSURE_SUCCESS (rv, rv);
@@ -758,7 +765,7 @@ GMyportalProtocolHandler::RenderSmartURL (GbSmartSite *b)
 {
 	Write ("<form action=\"myportal:form:");
 	WriteHTMLEscape (gb_bookmark_get_id (GB_BOOKMARK (b)));
-	Write ("\" method=\"GET\">");
+	Write ("\" method=\"get\"><p>");
 
 	gint num_inputs = gb_smart_site_get_num_fields (b);
 
@@ -778,7 +785,7 @@ GMyportalProtocolHandler::RenderSmartURL (GbSmartSite *b)
 		Write ("<input type=\"image\" src=\"" FILE_PREFIX "file://");
 		WriteHTMLEscape (GB_BOOKMARK(b)->pixmap_file);
 	}
-	Write ("\" /></form>\n");
+	Write ("\" /></p></form>\n");
 
 	return NS_OK;
 }
