@@ -21,6 +21,7 @@
 #include "print-dialog.h"
 #include "galeon-embed.h"
 #include "GaleonUtils.h"
+#include "AutoJSContextStack.h"
 
 #include <nsIPrintSettings.h>
 #include <nsCOMPtr.h>
@@ -49,6 +50,10 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent,
 	EmbedPrintInfo *info;
 	nsresult rv;
 
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	GtkWidget *gtkParent = GaleonUtils::FindGtkParent(parent);
 	GaleonEmbed *embed = GALEON_EMBED (GaleonUtils::FindEmbed (parent));
 
