@@ -20,7 +20,14 @@
 #include "config.h"
 #endif
 
+#define MOZILLA_INTERNAL_API
+#include <nscore.h>
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+
+#define MOZILLA_CLIENT
 #include "gtkmozembed.h"
+#undef MOZILLA_CLIENT
 #include "gul-string.h"
 #include "galeon-embed-utils.h"
 #include "galeon-embed.h"
@@ -44,11 +51,9 @@
 #include <nsIDOMHTMLLinkElement.h>
 #include <nsIStyleSheet.h>
 #include <nsIDOMDocument.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
 #include <nsIPrintSettings.h>
 #include <nsIDOMWindow.h>
+#include <nsIDOMKeyEvent.h>
 
 #include <math.h>
 
@@ -692,14 +697,7 @@ impl_reload (GaleonEmbed *embed,
 	g_free (location);
 
 	
-#ifndef HAVE_GTKMOZEMBED_BROKEN_RELOAD
 	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), reloadFlags);
-#else
-	/* Workaround for broken reload with frames, see mozilla bug
-	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	wrapper->ReloadPage (reloadFlags);
-#endif
 }
 
 static void
@@ -947,6 +945,26 @@ impl_scroll (GaleonEmbed *embed,
 	}
 }
 
+static void
+impl_scroll_page (GaleonEmbed *embed, 
+		  EmbedScrollDirection direction)
+{
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+
+	switch (direction)
+	{
+	case EMBED_SCROLL_UP:
+		wrapper->ScrollPageUp ();
+		break;
+	case EMBED_SCROLL_DOWN:
+		wrapper->ScrollPageDown ();
+		break;
+	case EMBED_SCROLL_LEFT:
+	case EMBED_SCROLL_RIGHT:
+		break;
+	}
+}
+
 static void
 impl_fine_scroll (GaleonEmbed *embed, int horiz, int vert)
 {
@@ -1041,32 +1059,6 @@ impl_print_preview_navigate (GaleonEmbed *embed,
 	return NS_SUCCEEDED(result) ? TRUE : FALSE;
 }
 
-static gboolean
-impl_find_next (GaleonEmbed *embed, 
-                gboolean backwards)
-{
-	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	nsresult result;
-        PRBool didFind;
-
-        result = wrapper->Find (backwards, &didFind);
-	
-	return didFind ? TRUE : FALSE;
-}
-
-
-static void
-impl_find_set_properties (GaleonEmbed *embed, 
-                          char *search_string,
-	                  gboolean case_sensitive,
-			  gboolean wrap_around)
-{
-	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-
-        wrapper->FindSetProperties (GulString (search_string).get(), 
-				    case_sensitive, wrap_around);
-}
-
 static gboolean
 impl_set_encoding (GaleonEmbed *embed,
 		   const char *charset)
@@ -1077,18 +1069,9 @@ impl_set_encoding (GaleonEmbed *embed,
 	rv = wrapper->SetForcedEncoding (charset);
 	if (NS_FAILED (rv)) return FALSE;
 
-#ifndef HAVE_GTKMOZEMBED_BROKEN_RELOAD
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
-#else
-	/* Roll our own reload function. Mozilla is broken
-	 * w.r.t framesets.
-	 *
-	 * See http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	wrapper->ReloadPage (GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
-#endif
- 
+
 	return TRUE;
 }
 
@@ -1392,6 +1375,30 @@ mozilla_embed_dom_mouse_down (GtkMozEmbed *embed, gpointer dom_event)
 						"ge_dom_mouse_down");
 }
 
+static gint
+mozilla_embed_dom_key_press (GtkMozEmbed *embed, gpointer dom_event)
+{
+	gint retval = FALSE;
+
+	if (dom_event == NULL) return FALSE;
+
+	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
+	NS_ENSURE_TRUE (ev, FALSE);
+
+	if (!EventContext::CheckKeyPress (ev)) return FALSE;
+
+	GdkEvent *event = gtk_get_current_event ();
+	if (event == NULL) return FALSE; /* shouldn't happen! */
+
+	g_return_val_if_fail (GDK_KEY_PRESS == event->type, FALSE);
+
+	g_signal_emit_by_name (embed, "ge-search-key-press", event, &retval);
+
+	gdk_event_free (event);
+
+	return retval;
+}
+
 static void
 mozilla_embed_size_to (GtkMozEmbed *embed, gint width, gint height)
 {
@@ -1868,10 +1875,9 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->shistory_copy = impl_shistory_copy;
 	embed_class->shistory_clear = impl_shistory_clear;
 	embed_class->scroll = impl_scroll;
+	embed_class->scroll_page = impl_scroll_page;
 	embed_class->fine_scroll = impl_fine_scroll;
 	embed_class->get_security_level = impl_get_security_level;
-	embed_class->find_set_properties = impl_find_set_properties;
-	embed_class->find_next = impl_find_next;
 	embed_class->set_encoding = impl_set_encoding;
 	embed_class->get_encoding_info = impl_get_encoding_info;
 	embed_class->has_modified_forms = impl_has_modified_forms;
@@ -1929,6 +1935,7 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 	embed_class->destroy_brsr    = mozilla_embed_destroy_brsr;
 	embed_class->dom_mouse_click = mozilla_embed_dom_mouse_click;
 	embed_class->dom_mouse_down  = mozilla_embed_dom_mouse_down;
+	embed_class->dom_key_press   = mozilla_embed_dom_key_press;
 	embed_class->size_to         = mozilla_embed_size_to;
 	embed_class->new_window      = mozilla_embed_new_window;
 	embed_class->security_change = mozilla_embed_security_change;
