@@ -49,6 +49,10 @@ security_icon_clicked_cb (GtkWidget *widget, Statusbar *s);
 static GObjectClass *parent_class = NULL;
 static GtkTooltips *_statusbar_tooltips = NULL;
 
+#define STATUSBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_STATUSBAR, StatusbarPrivate))
+
+
 struct StatusbarPrivate
 {
 	gboolean visibility;
@@ -121,6 +125,8 @@ statusbar_class_init (StatusbarClass *klass)
 			g_cclosure_marshal_VOID__UINT,
 			G_TYPE_NONE, 1,
 			G_TYPE_UINT);
+
+	g_type_class_add_private (klass, sizeof (StatusbarPrivate));
 }
 
 static GtkWidget *
@@ -213,8 +219,6 @@ create_statusbar_security_icon (Statusbar *s)
 
 	gtk_box_pack_end (GTK_BOX(s), frame, FALSE, FALSE, 0);
 	
-	statusbar_set_security_state (s, FALSE, NULL);
-	
         gtk_widget_show_all (frame);
 }
 
@@ -266,7 +270,7 @@ statusbar_init (Statusbar *t)
 	g_object_ref (_statusbar_tooltips);
 	gtk_object_sink (GTK_OBJECT (_statusbar_tooltips));
 
-        t->priv = g_new0 (StatusbarPrivate, 1);
+        t->priv = STATUSBAR_GET_PRIVATE (t);
 	t->priv->visibility = TRUE;
 }
 
@@ -285,7 +289,6 @@ statusbar_finalize (GObject *object)
 	g_object_unref (s->priv->offline_action);
 	
 	g_free (s->priv->popup_uri);
-        g_free (s->priv);
 
 	g_object_unref (_statusbar_tooltips);
 
@@ -297,7 +300,7 @@ statusbar_new (GtkToggleAction *offline_action)
 {
 	Statusbar *s;
 
-	s = STATUSBAR (g_object_new (STATUSBAR_TYPE, NULL));
+	s = STATUSBAR (g_object_new (TYPE_STATUSBAR, NULL));
 
 	create_statusbar_offline_icon (s, offline_action);
 	create_statusbar_security_icon (s);
@@ -327,12 +330,11 @@ statusbar_set_visibility (Statusbar *t,
 
 void
 statusbar_set_security_state (Statusbar *t, 
-			      gboolean state,
+			      const char *stock_icon,
 			      const char *tooltip)
 {
         gtk_image_set_from_stock (GTK_IMAGE (t->priv->security_icon),
-				  state ? GALEON_STOCK_SECURE : GALEON_STOCK_INSECURE,
-				  GTK_ICON_SIZE_MENU);
+				  stock_icon, GTK_ICON_SIZE_MENU);
 
 	gtk_tooltips_set_tip (_statusbar_tooltips, t->priv->security_button, 
 			      tooltip, NULL);
