@@ -20,7 +20,7 @@
 #define GUL_GLADE_H
 
 #include <glib.h>
-#include <gtk/gtk.h>
+#include <gtk/gtksizegroup.h>
 #include <glade/glade-xml.h>
 
 typedef struct 
@@ -29,6 +29,11 @@ typedef struct
         GtkWidget **ptr;
 } WidgetLookup;
 
+typedef enum {
+	GUL_SENSITIVE_FOLLOW,
+	GUL_SENSITIVE_INVERT
+} GulSensitiveType;
+
 G_BEGIN_DECLS
 
 GtkWidget  *gul_glade_lookup_widget	(GtkWidget *widget, 
@@ -42,6 +47,19 @@ GladeXML   *gul_glade_widget_new 	(const char *file,
 void        gul_glade_lookup_widgets 	(GladeXML *gxml, 
 					 WidgetLookup *lookup_table);
 
+void        gul_glade_get_widgets       (GladeXML *gxml,
+		                         const char *widget_name, ...);
+
+void        gul_glade_size_group_widgets (GladeXML *gxml,
+		                          GtkSizeGroupMode mode,
+					  const char *widget_name, ...);
+
+void        gul_glade_group_sensitive    (GladeXML *gxml,
+		                          const char *master_name,
+					  GulSensitiveType type,
+					  const char *slave_name, ...);
+
+
 G_END_DECLS
 
 #endif
